'use strict';

var chalk = require('chalk');
var less = require('less');
var path = require('path');

var rLess = /\.less$/;

var compileLess = function compileLess(workdir, filename, filedata, callback) {
    less.render(filedata, {
        paths: [path.dirname(path.join(workdir, filename))]
    }, function (err, data) {
        if (err) {
            var column = err.column + 1;
            var line = err.line - 1;
            var codeLine = filedata.split('\n')[line];
            var offendingCharacter = undefined;
            var errMsg = undefined;

            if (column < codeLine.length) {
                offendingCharacter = chalk.red(codeLine[column - 1]);
            } else {
                offendingCharacter = '';
            }

            line += 1;
            errMsg = '[less]:' + line + ':' + column + ': ' + err.message;
            errMsg += '\n' + codeLine.substring(0, column - 1) + offendingCharacter + codeLine.substring(column);
            errMsg += '\n' + new Array(column).join(' ') + chalk.red('^');
            err = new Error(errMsg);
            callback(err);
        } else {
            callback(null, data.css);
        }
    });
};

module.exports = function (file, callback) {
    compileLess(file.get('workdir'), file.get('filename'), file.get('filedata'), function (err, filedata) {
        if (!err) {
            file.set('filename', file.get('filename').replace(rLess, '.css'));
            file.set('filedata', filedata);
        }
        callback(err);
    });
};
